@@ -1,91 +1,45 @@
 require_relative 'paint/version'
+require_relative 'paint/constants'
 require_relative 'paint/util'
 
-
 module Paint
   autoload :RGB_COLORS, 'paint/rgb_colors'
-  autoload :RGB_COLORS_ANSI, 'paint/rgb_colors_ansi'
-
-  NOTHING = "\033[0m".freeze
-
-  # Basic colors (often, the color differs when using the bright effect)
-  # Final color will be 30 + value for foreground and 40 + value for background
-  ANSI_COLORS = {
-    :black   => 0,
-    :red     => 1,
-    :green   => 2,
-    :yellow  => 3,
-    :blue    => 4,
-    :magenta => 5,
-    :cyan    => 6,
-    :white   => 7,
-    :default => 9,
-  }.freeze
-
-  # Terminal effects - most of them are not supported ;)
-  # See http://en.wikipedia.org/wiki/ANSI_escape_code
-  ANSI_EFFECTS = {
-    :reset         => 0,  :nothing         => 0,  # usually supported
-    :bright        => 1,  :bold            => 1,  # usually supported
-    :faint         => 2,
-    :italic        => 3,
-    :underline     => 4,                          # usually supported
-    :blink         => 5,  :slow_blink      => 5,
-    :rapid_blink   => 6,
-    :inverse       => 7,  :swap            => 7,  # usually supported
-    :conceal       => 8,  :hide            => 9,
-    :default_font  => 10,
-    :font0 => 10, :font1 => 11, :font2 => 12, :font3 => 13, :font4 => 14,
-    :font5 => 15, :font6 => 16, :font7 => 17, :font8 => 18, :font9 => 19,
-    :fraktur       => 20,
-    :bright_off    => 21, :bold_off        => 21, :double_underline => 21,
-    :clean         => 22,
-    :italic_off    => 23, :fraktur_off     => 23,
-    :underline_off => 24,
-    :blink_off     => 25,
-    :inverse_off   => 26, :positive        => 26,
-    :conceal_off   => 27, :show            => 27, :reveal           => 27,
-    :crossed_off   => 29, :crossed_out_off => 29,
-    :frame         => 51,
-    :encircle      => 52,
-    :overline      => 53,
-    :frame_off     => 54, :encircle_off    => 54,
-    :overline_off  => 55,
-  }.freeze
-
-  ANSI_COLORS_FOREGROUND = {
-    :black   => 30,
-    :red     => 31,
-    :green   => 32,
-    :yellow  => 33,
-    :blue    => 34,
-    :magenta => 35,
-    :cyan    => 36,
-    :white   => 37,
-    :default => 39,
-  }.freeze
-
-  ANSI_COLORS_BACKGROUND = {
-    :black   => 40,
-    :red     => 41,
-    :green   => 42,
-    :yellow  => 43,
-    :blue    => 44,
-    :magenta => 45,
-    :cyan    => 46,
-    :white   => 47,
-    :default => 49,
-  }.freeze
 
   class << self
-    # Takes a string and color options and colorizes the string
-    # See README.rdoc for details
+    # Takes a string and color options and colorizes the string, fast version without nesting
     def [](string, *options)
       return string.to_s if @mode.zero? || options.empty?
-      options = options.first if options.size == 1 && !options.first.respond_to?(:to_ary)
+      options = options[0] if options.size == 1 && !options[0].respond_to?(:to_ary)
       @cache[options] + string.to_s + NOTHING
     end
 
+    # Takes an array with string and color options and colorizes the string,
+    # extended version with nesting and substitution support
+    def %(paint_arguments, clear_color = NOTHING)
+      string, *options = paint_arguments
+      return string.to_s if options.empty?
+      substitutions = options.pop if options[-1].is_a?(Hash)
+      options = options[0] if options.size == 1 && !options[0].respond_to?(:to_ary)
+      current_color = @cache[options]
+
+      # Substitutions & Nesting
+      if substitutions
+        substitutions.each{ |key, value|
+          string.gsub!(
+            "%{#{key}}",
+            (value.is_a?(Array) ? self%[value, current_color] : value.to_s)
+          )
+        }
+      end
+
+      # Wrap string (if Paint.mode > 0)
+      if @mode.zero?
+        string.to_s
+      else
+        current_color + string.to_s + clear_color
+      end
+    end
+
     # Transforms options into the desired color. Used by @cache
     def color(*options)
       return '' if @mode.zero? || options.empty?
@@ -114,8 +68,8 @@ module Paint
           end
 
         when ::String
-          if option =~ /^#?(?:[0-9a-f]{3}){1,2}$/i
-            mix << hex(option, color_seen)
+          if option =~ /\A#?(?<hex_color>[[:xdigit:]]{3}{1,2})\z/ # 3 or 6 hex chars
+            mix << rgb_hex($~[:hex_color], color_seen)
             color_seen = :set
           else
             mix << rgb_name(option, color_seen)
@@ -146,14 +100,28 @@ module Paint
 
     # This variable influences the color code generation
     # Currently supported values:
-    # * 256    - 256 colors
-    # * 16     - only ansi colors and bright effect
-    # * 8      - only ansi colors
-    # * 0      - no colorization!
+    # * 0xFFFFFF  - 24-bit (~16 million) colors, aka truecolor
+    # * 256       - 256 colors
+    # * 16        - only ansi colors and bright effect
+    # * 8         - only ansi colors
+    # * 0         - no colorization!
     attr_reader :mode
-    def mode=(val) @cache.clear; @mode = val end
+    def mode=(val)
+      @cache.clear
 
-    # Adds ansi sequence
+      case val
+      when 0, 8, 16, 256, TRUE_COLOR
+        @mode = val
+      when TrueClass
+        @mode = TRUE_COLOR
+      when nil
+        @mode = 0
+      else
+        raise ArgumentError, "Cannot set paint mode to value <#{val}>, possible values are: 0xFFFFFF (true), 256, 16, 8, 0 (nil)"
+      end
+    end
+
+    # Adds ANSI sequence
     def wrap(*ansi_codes)
       "\033[" + ansi_codes*";" + "m"
     end
@@ -163,50 +131,52 @@ module Paint
       (background ? 40 : 30) + ANSI_COLORS[color_name]
     end
 
-    # Creates a 256-compatible color from rgb values
+    # If not true_color, creates a 256-compatible color from rgb values,
+    # otherwise, an exact 24-bit color
     def rgb(red, green, blue, background = false)
-      if @mode == 8 || @mode == 16
-        "#{background ? 4 : 3}#{rgb_like_value(red, green, blue, @mode == 16)}"
-      else
-        "#{background ? 48 : 38}#{rgb_value(red, green, blue)}"
+      case @mode
+      when 8
+        "#{background ? 4 : 3}#{rgb_to_ansi(red, green, blue, false)}"
+      when 16
+        "#{background ? 4 : 3}#{rgb_to_ansi(red, green, blue, true)}"
+      when 256
+        "#{background ? 48 : 38}#{rgb_to_256(red, green, blue)}"
+      when TRUE_COLOR
+        "#{background ? 48 : 38}#{rgb_true(red, green, blue)}"
       end
     end
 
-    # Creates 256-compatible color from a html-like color string
-    def hex(source, background = false)
-      string = source.tr '#',''
-      color_code = if string.size == 6
-        string.each_char.each_slice(2).map{ |hex_color| hex_color.join.to_i(16) }
-      else
-        string.each_char.map{ |hex_color_half| (hex_color_half*2).to_i(16) }
+    # Creates RGB color from a HTML-like color definition string
+    def rgb_hex(string, background = false)
+      case string.size
+      when 6
+        color_code = string.each_char.each_slice(2).map{ |hex_color| hex_color.join.to_i(16) }
+      when 3
+        color_code = string.each_char.map{ |hex_color_half| (hex_color_half*2).to_i(16) }
       end
       rgb(*[*color_code, background])
     end
 
-    # Creates a 256-color from a name found in Paint::RGB_COLORS (based on rgb.txt)
+    # Creates a RGB from a name found in Paint::RGB_COLORS (based on rgb.txt)
     def rgb_name(color_name, background = false)
       if color_code = RGB_COLORS[color_name]
         rgb(*[*color_code, background])
       end
     end
 
-    # Creates a random ansi color
-    def random(background = false)
-      (background ? 40 : 30) + rand(8)
-    end
-
     # Creates the specified effect by looking it up in Paint::ANSI_EFFECTS
     def effect(effect_name)
       ANSI_EFFECTS[effect_name]
     end
 
     # Determine supported colors
+    # Note: there's no reliable test for 24-bit color support
     def detect_mode
       if RbConfig::CONFIG['host_os'] =~ /mswin|mingw/ # windows
         if ENV['ANSICON']
           16
         elsif ENV['ConEmuANSI'] == 'ON'
-          256
+          TRUE_COLOR
         else
           0
         end
@@ -224,9 +194,17 @@ module Paint
 
     private
 
-    # Returns nearest supported 256-color an rgb value, without fore-/background information
+    # Returns 24-bit color value (see https://gist.github.com/XVilka/8346728)
+    # in ANSI escape sequnce format, without fore-/background information
+    def rgb_true(red, green, blue)
+      ";2;#{red};#{green};#{blue}"
+    end
+
+    # Returns closest supported 256-color an RGB value, without fore-/background information
     # Inspired by the rainbow gem
-    def rgb_value(red, green, blue)
+    def rgb_to_256(red, green, blue, approx = true)
+      return ";2;#{red};#{green};#{blue}" unless approx
+
       gray_possible = true
       sep = 42.5
 
@@ -247,23 +225,22 @@ module Paint
       end
     end
 
-    # Returns ansi color matching an rgb value, without fore-/background information
+    # Returns best ANSI color matching an RGB value, without fore-/background information
     # See http://mail.python.org/pipermail/python-list/2008-December/1150496.html
-    def rgb_like_value(red, green, blue, use_bright = false)
+    def rgb_to_ansi(red, green, blue, use_bright = false)
       color_pool =  RGB_COLORS_ANSI.values
       color_pool += RGB_COLORS_ANSI_BRIGHT.values if use_bright
 
-      ansi_color_rgb = color_pool.min_by{ |col| distance([red, green, blue],col) }
-      key_method = RUBY_VERSION < "1.9" ? :index : :key
-      if ansi_color = RGB_COLORS_ANSI.send(key_method, ansi_color_rgb)
+      ansi_color_rgb = color_pool.min_by{ |col| rgb_color_distance([red, green, blue],col) }
+      if ansi_color = RGB_COLORS_ANSI.key(ansi_color_rgb)
         ANSI_COLORS[ansi_color]
       else
-        ansi_color = RGB_COLORS_ANSI_BRIGHT.send(key_method, ansi_color_rgb)
+        ansi_color = RGB_COLORS_ANSI_BRIGHT.key(ansi_color_rgb)
         "#{ANSI_COLORS[ansi_color]};1"
       end
     end
 
-    def distance(rgb1, rgb2)
+    def rgb_color_distance(rgb1, rgb2)
       rgb1.zip(rgb2).inject(0){ |acc, (cur1, cur2)|
         acc + (cur1 - cur2)**2
       }
@@ -274,5 +251,3 @@ module Paint
   @mode  = detect_mode
   @cache = Hash.new{ |h, k| h[k] = color(*k) }
 end
-
-# J-_-L
